//ImageB01.pov for POV-Ray 3.6: After the Storm  (c) 2004 Ch. Gerber

global_settings {assumed_gamma 2.6 max_trace_level 20}

#declare HQ = 1;
#declare blur = 0;  //try if your computer is *much* faster than a 3.6 GHz Intel P4 with 1GB RAM ...
#declare clouds = 1;
#declare moss = 1;
#declare plants = 1;
#declare wet = 1;
#declare globes = 1;
#declare towers = 1;
#declare insects = 1; 

#include "colors.inc"
#include "rand.inc"
#include "functions.inc"
#include "iso_csg.inc"
#include "mytextures.inc"
#include "myobjects.inc"

camera {
  location <0,10,-140>
  direction 1.5*z
  right 3/4*x
  look_at <0,275,1000>
  #if(blur = 1)
    aperture 0.7
    focal_point <0,15,10>
    blur_samples 100
    confidence 1/128
    variance 1/10000
  #end 
}

light_source {
  -10000*z 
  color rgb 1.5*<1.0,0.8,0.5>
  #if (HQ=1)
   area_light <1000,0,0>,<0,1000,0>,5,5 
   adaptive 1
   jitter
   circular
   orient
  #end 
  rotate <30,-50,0>
  parallel point_at <0,0,0>
  media_attenuation off
  media_interaction off
}

light_source {
  -10000*z 
  color rgb 0.4*<0.9,0.9,1.0> 
  rotate <45,10,0>
  parallel point_at <0,0,0>
  media_attenuation off
  media_interaction off
  shadowless
}

//--- sky ---
    
sky_sphere {
  pigment {
    gradient y 
    turbulence 0.2
    color_map { 
      [0.0 color rgb <1,0.9,0.375>/2]
      [0.8 color rgb <0.7,0.8,1>]
    }
    rotate 15*x
  }
}

fog {
  fog_type 2
  distance 1500
  color Gray50
  turbulence 0.7
  fog_offset 0
  fog_alt 125
}

rainbow {
  angle 26.5
  width 4
  arc_angle 100
  falloff_angle 30
  distance 1.0e7
  direction <-0.1,0,1>
  jitter 0.01
  color_map {
    [0.000  color rgbft <1.0,0.5,1.0,0.8,0.98>]
    [0.100  color rgbft <1.0,0.5,1.0,0.8,0.95>]
    [0.214  color rgbft <0.5,0.5,1.0,0.8,0.93>]
    [0.328  color rgbft <0.2,0.2,1.0,0.8,0.91>]
    [0.442  color rgbft <0.2,1.0,1.0,0.8,0.89>]
    [0.556  color rgbft <0.2,1.0,0.2,0.8,0.91>]
    [0.670  color rgbft <1.0,1.0,0.2,0.8,0.93>]
    [0.784  color rgbft <1.0,0.5,0.2,0.8,0.95>]
    [0.900  color rgbft <1.0,0.2,0.2,0.8,0.98>]
  }
}

#if (clouds = 1)

light_group {
  
  light_source {
   -93000000*z
    color rgb <1.0,0.8,0.6>*1.5
    rotate <30,-50,0>
    parallel point_at <0,0,0>
    media_attenuation on
    media_interaction on
  }
  
  #declare Rad = 2500000/(2*pi);

  difference {
    sphere {0, Rad}
    sphere {0, Rad-15000}
    material {
      texture {pigment {rgbf 1}}
      interior {
        media {
          scattering {2, 0.01/20 extinction 1}
          method 3
          #if (HQ=1) samples 20,20 #else samples 5,5 #end
          intervals 1
          density {
            wrinkles
            ramp_wave
            color_map {
              [0   rgb 0]
              [0.6 rgb 0.4]
              [1   rgb 0.7]
            }
            scale <10000,5000,9000>
            rotate y*25
            translate <1000,Rad-30000,0>
          }
          density {
            marble
            triangle_wave
            turbulence 1.5
            color_map {
              [0    rgb 0]
              [0.75 rgb 0.35]
              [1    rgb 0.7]
            }
            scale <10000,5000,7500>*15
            rotate -25*y
            translate <0,Rad-30000,0>
          }
        }
      }
    }
    hollow
    rotate <0,70,10>
    translate <0,32000-Rad,0>
  }
  
  global_lights off
}  

#end //clouds

//--- foreground hill 1 ---

#declare VegeObject = object {
  height_field {tga "range.tga" smooth translate <-0.5,0,-0.5>}
  scale <100,20,100> 
  texture {GroundTex}
}

#declare VegeList = array[1] 
#declare VegeList[0] = object {MossPatch translate -0.5*y}
#declare VegeSeed = seed(5);
#declare VegeNumber = 6000;
#declare VegeMaxScale = 1.0; 
#declare VegeMinScale = 0.7;
#declare VegeAlign = on;
#declare VegeMinBox = <-50,-50>;
#declare VegeMaxBox = < 50, 50>;   

#declare Step = 2;
#declare MaxSlope = 45;
#declare MaxHeight = 50;
#declare MinHeight = 0;
#declare VegeSeed = seed(13);
#declare Pig = pigment {spotted color_map {[0 rgb 1][1 rgb 0]} scale 5}

union {
  object {VegeObject}
  #if (moss=1) #declare Moss = union {#include "vegetate.inc" texture {MossTex}} object {Moss} #end
  #if (plants=1) union {Shroomz(VegeObject,VegeMinBox,VegeMaxBox,Step,MaxSlope,MaxHeight,MinHeight,VegeSeed,Pig)} #end
  #if (moss=1 & wet=1) object {WetOnTop(VegeMinBox,VegeMaxBox,Moss,8000,0.7) material {WaterMat}} #end
  rotate 90*y
  translate 20*x 
}  

//--- foreground hill 2 ---

#declare VegeObject = object {
  height_field {tga "range.tga" smooth translate <-0.5,0,-0.5>}
  scale <100,15,100> 
  texture {GroundTex}
}

#declare VegeNumber = 1000;
#declare VegeMinBox = <-0,-50>;
#declare VegeMaxBox = <40,  5>;

union {
  object {VegeObject}
  #if (moss=1) #declare Moss = union {#include "vegetate.inc" texture {MossTex}} object {Moss} #end
  #if (plants=1) union {Shroomz(VegeObject,VegeMinBox,VegeMaxBox,Step,MaxSlope,MaxHeight,MinHeight,VegeSeed,Pig)} #end
  #if (moss=1 & wet=1) object {WetOnTop(VegeMinBox,VegeMaxBox,Moss,1300,0.7) material {WaterMat}} #end
  translate <-50,0,50> 
} 

//--- foreground hill 3 ---

#declare VegeObject = object {
  height_field {tga "range.tga" smooth translate <-0.5,0,-0.5>}
  scale <100,12,100> 
  texture {GroundTex}
}

#declare VegeNumber = 500;
#declare VegeMinBox = <-10,-30>;
#declare VegeMaxBox = < 20,  5>;

union {
  object {VegeObject}
  #if (moss=1) #declare Moss = union {#include "vegetate.inc" texture {MossTex}} object {Moss} #end
  #if (plants=1) union {Shroomz(VegeObject,VegeMinBox,VegeMaxBox,Step,MaxSlope,MaxHeight,MinHeight,VegeSeed,Pig)} #end
  #if (moss=1 & wet=1) object {WetOnTop(VegeMinBox,VegeMaxBox,Moss,650,0.7) material {WaterMat}} #end
  translate <-35,0,150> 
} 

//--- background hills ---

#declare VegeObject = object {
  height_field {
    tga "hills.tga" smooth
    scale <30000,3750,30000>
    texture {GroundTex}
  }
  translate <-15000,0,-500>
}

object {VegeObject}

#declare VegeList = array[8] 
#declare VegeList[0] = object {Bush() scale <200,100,200>}
#declare VegeList[1] = object {Bush() scale <200,100,200>}  
#declare VegeList[2] = object {Bush() scale <200,100,200>}  
#declare VegeList[3] = object {Bush() scale <200,100,200>}  
#declare VegeList[4] = object {Bush() scale <200,100,200>}
#declare VegeList[5] = object {Bush() scale <200,100,200>}
#declare VegeList[6] = object {Bush() scale <200,100,200>}
#declare VegeList[7] = object {Bush() scale <200,100,200>}
#declare VegeSeed = seed(41); 
#declare VegeNumber = 8000;
#declare VegeMaxScale = 0.8; 
#declare VegeMinScale = 1.2;
#declare VegeAlign = on;
#declare VegeWaterLevel = 10;
#declare VegeMinBox = <-7000,750>;
#declare VegeMaxBox = <4500,18000>;   

#if (plants=1) #include "vegetate.inc" #end

#declare VegeList = array[1]
#declare VegeList[0] = object {Globe scale 12.5}
#declare VegeSeed = seed(13);
#declare VegeNumber = 120;
#declare VegeMaxSlope = 45;
#declare VegeWaterLevel = 0.3;

#if (globes=1) union {#include "vegetate.inc"} #end

#include "tree.inc"

#declare VegeList = array[1]
#declare VegeList[0] = object {TREE scale 450}
#declare VegeSeed = seed(53);
#declare VegeNumber = 240;
#declare VegeMaxSlope = 45;
#declare VegeAlign = off;
#declare VegeWaterLevel = 0.3;

#if(plants=1) #include "vegetate.inc" #end

//--- towers ---

object {Tower scale 2 translate <20,-0.5,150>}
object {Tower scale 2 rotate 30*y translate <-35,-5,450>}

#if (towers = 1)

#declare VegeList = array[1]
#declare VegeList[0] = object {TowerFast scale 10 translate -5*y}
#declare VegeSeed = seed(69);
#declare VegeNumber = 30;
#declare VegeMaxSlope = 45;
#declare VegeAlign = off;
#declare VegeWaterLevel = 5;

#include "vegetate.inc"

#end //towers

//--- inscets ---

#if (insects = 1)

#declare BoxMin = <-130, 10,-100>;
#declare BoxMax = < 130,150, 250>;
#declare InsectSeed = seed(32);
#declare nInsects = 30;
#declare i = 0;

#while (i < nInsects)
  object {
    DragonFly
    scale RRand(1.5,2,InsectSeed)
    rotate <RRand(-5,5,InsectSeed),360*rand(InsectSeed),RRand(-5,5,InsectSeed)>
    translate <rand(InsectSeed),rand(InsectSeed),rand(InsectSeed)>*(BoxMax-BoxMin)+BoxMin
  }
  #declare i = i + 1;
#end

object {DragonFly scale RRand(1.5,2,InsectSeed) rotate <RRand(-5,5,InsectSeed),180*rand(InsectSeed),RRand(-5,5,InsectSeed)> translate < 20,28, -5>}
object {DragonFly scale RRand(1.5,2,InsectSeed) rotate <RRand(-5,5,InsectSeed),360*rand(InsectSeed),RRand(-5,5,InsectSeed)> translate < 10,10,-50>}
object {DragonFly scale RRand(1.5,2,InsectSeed) rotate <RRand(-5,5,InsectSeed),360*rand(InsectSeed),RRand(-5,5,InsectSeed)> translate < 10,25,-10>}
object {DragonFly scale RRand(1.5,2,InsectSeed) rotate <RRand(-5,5,InsectSeed),360*rand(InsectSeed),RRand(-5,5,InsectSeed)> translate <-10,15,-15>}
object {DragonFly scale RRand(1.5,2,InsectSeed) rotate <RRand(-5,5,InsectSeed),360*rand(InsectSeed),RRand(-5,5,InsectSeed)> translate <  5,30,  0>}
object {DragonFly scale RRand(1.5,2,InsectSeed) rotate <RRand(-5,5,InsectSeed),360*rand(InsectSeed),RRand(-5,5,InsectSeed)> translate <-15,15,-50>}
object {DragonFly scale RRand(1.5,2,InsectSeed) rotate <RRand(-5,5,InsectSeed),360*rand(InsectSeed),RRand(-5,5,InsectSeed)> translate < -5, 5,-60>}
    
#end //insects   
