#declare R = seed(2004); //global random

//--- plants and plant positioning ---

#macro mAlign(P1,P2)
//returns a matrix operation that aligns an object or texture along P2-P1
//the object is translated to P1
  #local yV1 = vnormalize(P2-P1);
  #local xV1 = vnormalize(vcross(yV1,z));
  #local zV1 = vcross(xV1,yV1);
  matrix <xV1.x,xV1.y,xV1.z,yV1.x,yV1.y,yV1.z,zV1.x,zV1.y,zV1.z,P1.x,P1.y,P1.z>
#end //mAlign   

#macro Spore(p0,dIni,Rad,Pert,nSegs,StemTex,BulbTex)
//pO      = location of the spore
//dIni    = initial direction of growth, general growth direction is y
//Rad     = radius at basis
//Pert    = perturbance of growth
//nSegs   = number of segments
//StemTex = texture of the stem
//BulbTex = texture of the bulb
  #local pts = array[nSegs+1]
  #local pts[0] = p0;
  #local dir = dIni;
  #local i = 1;
  #while (i<nSegs+1)
    #local s = (1/nSegs);
    #local disp = 4*s*(1-s)*Pert;
    #local pts[i] = pts[i-1]+dir;
    #local dir = vnormalize(pts[i]-pts[i-1]-<(rand(R)-0.5)*disp,(rand(R)-0.5)*disp,(rand(R)-0.5)*disp>+(dIni*(nSegs-i)/5+y*i)/nSegs);
    #local i = i + 1;
  #end                               
  blob {
    threshold 0.5
    sphere {pts[0],Rad,1 texture {StemTex mAlign(pts[0],pts[1])}}
    #local i = 1;
    #while (i <= nSegs)
      #local r1 = Rad - i*Rad/(2*nSegs);
      #if (i = nSegs)
        #local r1 = r1*3;
        sphere {pts[i],r1,1 texture {BulbTex mAlign(pts[i-1],pts[i])}}
      #else
        cylinder {pts[i-1],pts[i],r1,1 texture {StemTex mAlign(pts[i-1],pts[i])}}
        sphere {pts[i-1],r1,-1 texture {StemTex mAlign(pts[i-1],pts[i])}}
      #end  
      #local i = i + 1;
    #end
  }
#end //Spore

#macro VRand_On_Obj(Obj,RS)
//returns point randomly selected on the surface of any object
  #local Mn = min_extent(Obj);
  #local Mx = max_extent(Obj);
  #local Pta = VRand_On_Sphere(RS)*vlength(Mx-Mn) + (Mx+Mn)/2;
  #local j = 0;
  #local Norm = <0,0,0>;
  #while(vlength(Norm) = 0 & j<100000)
    #local Ptb  = VRand_In_Obj(Obj,RS);
    #local Pt = trace (Obj,Ptb,Pta,Norm);
    #local j = j + 1;
  #end
  (Pt)
#end  //VRand_On_Obj

#macro WetOnSurface(obj,dropcount,dropsize)
//returns dropcount drops of the size dropsize on the surface of any object
  blob {
    threshold 1
    #declare rseed = seed(3);
    #local i = dropcount;
    #while(i >= 0)
      sphere {VRand_On_Obj(obj,rseed),RRand(0.6,1.0,rseed)*dropsize,1}
      #local i = i - 1;
    #end
  }
#end //WetOnSurface 

#macro WetOnTop(SquareMin,SquareMax,obj,dropcount,dropsize) 
  blob {
    threshold 1
    #declare rseed = seed(5);
    #local i = dropcount;
    #while(i >= 0)
      #local pos = <rand(rseed),rand(rseed)>*(SquareMax-SquareMin)+SquareMin;
      #local spos = <pos.x,100,pos.y>;
      sphere {trace(obj,spos,<0,-1,0>),RRand(0.6,1.0,rseed)*dropsize,1}
      #local i = i - 1;
    #end
  }  
#end //WetOnTop 

#declare fn1 = function {
  pigment {
    crackle form <1,0,0>
    color_map {
      [0.1 color rgb 1]
      [0.6 color rgb 0.1]
      [0.9 color rgb 0]
    }
    scale 2.5 
  }     
}

#declare fn2 = function {
  pigment {
    spotted
    color_map {
      [0.0 color rgb 1]
      [1.0 color rgb 0]
    }
    scale 7.5 
  }     
}

#declare Spore2 = object {
  isosurface {
    function {f_sphere(x,y,z,10)-fn1(x,y,z).gray*0.5+fn2(x,y,z).gray*2}
    max_gradient 1.5
    contained_by {sphere {0,12}}
  }
  texture {Spore2Tex}
}

#declare Spore2Wet = union {
  object {Spore2}
  object {WetOnSurface(Spore2,20,3.5) material {WaterMat}}
}

#declare MossPatch = isosurface {
  function {y+2-4*f_noise3d(30*x,y,30*z)}
  contained_by {sphere {0,3}}
  max_gradient 10
  accuracy 0.001
  open
}

#macro Bush()
//returns a bush made of blobs
  #ifndef (R) #local R=seed(42); #end
  union {
    #local Blobbi = 0;
    #while (Blobbi<5)
      #local Blobbi = Blobbi + 1;
      blob {
        threshold 0.65
        #local Buschi = 0;
        #while (Buschi<rand(R)*5+5)
          #local Buschi = Buschi + 1;
          #local Buschteilrad = 1 - pow(rand(R),3);
          sphere {
            <Buschteilrad,0,0>,0.5,1
            rotate 360*<rand(R),rand(R),rand(R)>
            scale <1-rand(R)/2,1-rand(R)/2,1-rand(R)/2>
          }
        #end  
        pigment {
          radial
          turbulence 1 octaves 6 omega 8
          color_map {
            [0.0 color rgb <0.3,0.4,0.05> transmit 1]
            [0.5 color rgb <RRand(0.2,0.4,R),RRand(0.3,0.5,R),RRand(0.0,0.1,R)> transmit 1]
            [0.5+rand(R)*0.2 color rgb <RRand(0.2,0.4,R),RRand(0.3,0.5,R),RRand(0.0,0.1,R)>]
            [1  color rgb <RRand(0.1,0.3,R),RRand(0.4,0.6,R),RRand(0.0,0.1,R)>]
          }
          scale 0.3
        }
        finish {ambient 0.0 diffuse 0.6 phong 0.2 phong_size 10}
      }
    #end
  }
#end  

#macro SLOPETest(Norm,MaxSlope)
  #declare STest = false;
  #if (vlength(Norm)!=0)
    #declare STest = true;
    #local Test = acos(vdot(Norm,y)/(vlength(Norm)*vlength(y)))*(180/pi);
    #if (Test>MaxSlope) #if (Test<180-MaxSlope) #declare STest = false; #end #end
  #end  
#end

#macro Shroomz(VegeObject,VegeMinBox,VegeMaxBox,Step,MaxSlope,MaxHeight,MinHeight,VegeSeed,Pig)
  #local Norm = <0,0,0>;
  #local Xpos = VegeMinBox.x;
  #local Zpos = VegeMinBox.y;
  #while (Xpos<VegeMaxBox.x)
    #while (Zpos<VegeMaxBox.y)
      #local SPos = <Xpos+RRand(-0.2*Step,0.2*Step,VegeSeed),100,Zpos+RRand(-0.2*Step,0.2*Step,VegeSeed)>;
      #local PP = eval_pigment(Pig,<Xpos,0,Zpos>);
      #local XX = PP.x;
      #local Inter = trace (VegeObject,SPos,<0,-1,0>,Norm);
      SLOPETest(Norm,MaxSlope)
      #if (XX>0.5 & STest=true & Inter.y<MaxHeight & Inter.y>MinHeight) 
        object {
          Spore(<0,0,0>,Norm,RRand(2,5,VegeSeed),RRand(6,8,VegeSeed),int(RRand(12,44,VegeSeed)),Spore1StemTex,Spore1BulbTex)
          scale 0.15
          translate Inter    
        }  
      #end 
      #local Zpos = Zpos + Step;
    #end           
    #local Zpos = VegeMinBox.y;
    #local Xpos = Xpos + Step; 
  #end
  #local Norm = <0,0,0>;
  #local Xpos = VegeMinBox.x;
  #local Zpos = VegeMinBox.y;
  #local Step2 = Step*3;
  #if(wet=1) #local SporeX = object {Spore2Wet} #else #local SporeX = object {Spore2} #end
  #while (Xpos<VegeMaxBox.x)
    #while (Zpos<VegeMaxBox.y)
      #local SPos = <Xpos+RRand(-0.2*Step2,0.2*Step2,VegeSeed),100,Zpos+RRand(-0.2*Step2,0.2*Step2,VegeSeed)>;
      #local PP = eval_pigment(Pig,<Xpos,0,Zpos>);
      #local XX = PP.x;
      #local Inter = trace (VegeObject,SPos,<0,-1,0>,Norm);
      SLOPETest(Norm,MaxSlope)
      #if (XX<0.4 & STest=true & Inter.y<MaxHeight & Inter.y>MinHeight) 
        object {
          SporeX
          scale RRand(0.15,0.3,VegeSeed)
          rotate 360*VegeSeed
          translate Inter + 0.5*y //! 0.4*y
        }  
      #end 
      #local Zpos = Zpos + Step2;
    #end           
    #local Zpos = VegeMinBox.y;
    #local Xpos = Xpos + Step2; 
  #end
#end

//--- architecture ---

#declare PointArray1 = array[12] { 
  <4.0, -1.0>,
  <4.0,  0.0>,
  <4.0, 35.0>,
  <4.2, 36.5>,
  <5.3, 39.0>,
  <5.9, 41.0>,
  <6.0, 43.0>,
  <5.0, 46.0>,
  <3.1, 49.0>,
  <1.1, 53.0>,
  <0.1, 58.0>,
  <0.1, 59.0>
}

#declare PointArray2 = array[12] { 
  <3.7, -1.0>,
  <3.7,  0.0>,
  <3.7, 35.0>,
  <3.9, 36.5>,
  <5.0, 39.0>,
  <5.6, 41.0>,
  <5.7, 43.0>,
  <4.7, 46.0>,
  <2.8, 49.0>,
  <0.8, 53.0>,
  <0.1, 57.5>,
  <0.1, 58.5>
}

#declare fn3 = IC_Sor (PointArray1, IC_Close)

#declare fn4 = IC_Sor (PointArray2, IC_Close)

#declare fn5 = IC_Difference2 (fn3,fn4)

#declare fn6 = function {
  pigment {
    crackle
    lambda 1
    octaves 2
    color_map {
      [0.00 Black]
      [0.15 Black]
      [0.17 White]
      [1.00 White]
    }
  }  
}  

#declare Tower = union {
  isosurface {
    function {fn5(x,y,z) + fn6(x,y,z).gray}
    contained_by {box {<-6.5,-1,-6.5>,<6.5,59,6.5>}}
    #if(HQ=1) max_gradient 15 #else max_gradient 3 #end //max_gradient should be much higher but that would be by far too slow
    texture {MetalTex scale 2}
  }
  sphere { 0, 1 hollow
    texture {pigment {color rgbt 1}}
    interior {
      media {
        emission 0.15
        method 3
        intervals 1
        samples 5,5
        density {
          spherical
          color_map {
            [0.0 color rgb <0,0,0>]
            [0.5 color rgb <0.3,0,0>]
            [1.0 color rgb <1,1,0>]
          }
        }
      }
    }
    scale 6  
    translate 43*y   
  }
  sphere { 0, 1 hollow
    texture {pigment {color rgbt 1}}
    interior {
      media {
        emission 0.2
        method 3
        intervals 1
        samples 5,5
        density {
          spherical
          color_map {
            [0.0 color rgb <0,0,0>]
            [0.5 color rgb <0,0,0.2>]
            [1.0 color rgb <0.3,0,0.7>]
          }
        }
      }
    }
    scale <4,9,4>  
    translate 30*y   
  }
  sphere { 0, 1 hollow
    texture {pigment {color rgbt 1}}
    interior {
      media {
        emission 0.2
        method 3
        intervals 1
        samples 5,5
        density {
          spherical
          color_map {
            [0.0 color rgb <0,0,0>]
            [0.5 color rgb <0,0,0.2>]
            [1.0 color rgb <0.3,0,0.7>]
          }
        }
      }
    }
    scale <4,9,4>  
    translate 20*y   
  }
}    

#declare TowerFast = union {
  object {
    sor {
      12,
      <4.0, -1.0>,
      <4.0,  0.0>,
      <4.0, 35.0>,
      <4.2, 36.5>,
      <5.3, 39.0>,
      <5.9, 41.0>,
      <6.0, 43.0>,
      <5.0, 46.0>,
      <3.1, 49.0>,
      <1.1, 53.0>,
      <0.1, 58.0>,
      <0.1, 59.0>
      sturm
    }
    texture {
      crackle
      lambda 1
      octaves 2
      texture_map {
        [0.0 MetalTex]
        [0.2 MetalTex]
        [0.2 pigment {rgbt 1} finish {ambient 0 diffuse 0}]
        [1.0 pigment {rgbt 1} finish {ambient 0 diffuse 0}]
      }
    }
    hollow
  }  
  sphere { 0, 1 hollow
    texture {pigment {color rgbt 1}}
    interior {
      media {
        emission 0.1
        method 3
        intervals 1
        samples 5,5
        density {
          spherical
          color_map {
            [0.0 color rgb <0,0,0>]
            [0.5 color rgb <0.3,0,0>]
            [1.0 color rgb <1,1,0>]
          }
        }
      }
    }
    scale 6  
    translate 43*y   
  }
  sphere { 0, 1 hollow
    texture {pigment {color rgbt 1}}
    interior {
      media {
        emission 0.13
        method 3
        intervals 1
        samples 5,5
        density {
          spherical
          color_map {
            [0.0 color rgb <0,0,0>]
            [0.5 color rgb <0,0,0.2>]
            [1.0 color rgb <0.3,0,0.7>]
          }
        }
      }
    }
    scale <4,9,4>  
    translate 30*y   
  }
  sphere { 0, 1 hollow
    texture {pigment {color rgbt 1}}
    interior {
      media {
        emission 0.13
        method 3
        intervals 1
        samples 5,5
        density {
          spherical
          color_map {
            [0.0 color rgb <0,0,0>]
            [0.5 color rgb <0,0,0.2>]
            [1.0 color rgb <0.3,0,0.7>]
          }
        }
      }
    }
    scale <4,9,4>  
    translate 20*y   
  }  
}

#declare Globe = union {
  object {#include "globe.inc" texture {GlobeTex}}
  sphere {
    0,1 hollow
    pigment {rgbt 1}
    interior {
      media {
        emission 0.09
        method 3
        intervals 1
        samples 5,5
        density {
          spherical
          color_map {
            [0.0 color rgb <0,0,0>]
            [0.1 color rgb <1,0.2,0>]
          }
        }
      }  
    }
    scale 4.3      
  }
  rotate -90*x 
  scale 3
}

//--- dragonfly ---

#macro Wing(WingCount)
  object {
    height_field {gif "Wings.gif" smooth water_level 0.00025 translate -1*x}
    #if (WingCount=0) texture {WingTex0} #end
    #if (WingCount=1) texture {WingTex1} #end
    #if (WingCount=2) texture {WingTex2} #end
    #if (WingCount=3) texture {WingTex3} #end
    #if (WingCount=4) texture {WingTex4} #end
    scale <1.5,0.01,1> 
  }
#end  

#declare Wings = union {
  #local WingCount = 0;
  #while (WingCount<5)
    union {
      object {Wing(WingCount) rotate (2-WingCount*5)*z translate -0.07*x}
      object {Wing(WingCount) scale <-1,1,1> rotate(-2+WingCount*5)*z translate 0.07*x}
    }
    #local WingCount = WingCount + 1;
  #end
  scale 12
  rotate <-90,0,-90>
  translate -18.5*y
}

#declare Segment = difference {
  cylinder {-1*y,1*y,1}
  torus {2.5,1.75}
  rotate 90*z
  scale <2,1,1>
}

#declare FSegment = union{
  object {Segment}
  object {sphere {0,1 scale <0.5,1,1> translate -2*x}}
  object {sphere {0,1 scale <0.5,1,1> translate  2*x}}
}

#declare Claws = union {
  #local Bend = 5;
  #local Start_Rad = 0.15;
  #local Inc = 0.0;
  #local Angle = 0;
  #local Count = 0;
  #while(Count<12)
    sphere {0,Start_Rad-Inc translate Bend*y rotate Angle*z}
    #local Count = Count +1;
    #local Angle = Angle +1.25;
    #local Inc = Inc + 0.01;
  #end 
}  
        
#declare Tail = union {
  object {FSegment rotate 6*y}
  object {FSegment scale <1.25,1.1,1.1> rotate 1*y translate  -4.9*x rotate  9*y}
  object {FSegment scale <1.0, 1.0,1.0> rotate 4*y translate -10.0*x rotate 11*y}
  object {FSegment scale <0.8, 0.9,0.9> rotate 6*y translate -14.0*x rotate 13*y}
  object {FSegment scale <0.6, 0.8,0.8> rotate 4*y translate -17.0*x rotate 14*y}
  object {FSegment scale <0.5, 0.7,0.7> rotate 2*y translate -19.5*x rotate 14.5*y}
  object {sphere {0,0.7 translate -21*x} rotate 15*y}
  scale 0.5
}

#declare Torso = union {
 sphere {0,1 scale <3,1,1> translate <4, 0.0,0>}
 sphere {0,1 scale <2,1,1> translate <5, 0.3,0>}
 sphere {0,1 scale <2,1,1> translate <5,-0.3,0>}
}

#declare Head = union {
  sphere {0,0.75 scale <1,1.5,1> translate<7,0,0> texture {pigment {radial} finish {ambient 0 diffuse 0.45 specular 1} normal {granite 0.3 scale 0.001}}}
  sphere {0,0.5 scale <1.2,1,1> translate<7.25, 0.75,-0.25> texture {pigment {rgb <0,1,2>} finish {ambient 0 diffuse 0.45 specular 1} normal {crackle 2 scale 0.1}}}
  sphere {0,0.5 scale <1.2,1,1> translate<7.25,-0.75,-0.25> texture {pigment {rgb <0,1,2>} finish {ambient 0 diffuse 0.45 specular 1} normal {crackle 2 scale 0.1}}}
}
        
#declare DragonFly = union {
  object {Wings translate <-2,18.5,-0.5>}
  object {Head  texture {pigment {radial} finish {ambient 0 diffuse 0.45 specular 1}}}
  object {Torso texture {pigment {radial} finish {ambient 0 diffuse 0.45 specular 1} normal {granite 0.3 scale 0.001}}}                        
  object {Tail rotate 0*y translate 1*x texture {pigment {radial rotate 150*y } finish {ambient 0 diffuse  0.45 specular 1} normal {granite 0.3 scale 0.001}}}
  object {Claws rotate -15*y rotate -180*x rotate -15*z translate <-8.15, 5.025,2.8> texture {pigment {radial} finish {ambient 0 diffuse 0.45 specular 1}}}
  object {Claws rotate  15*y               rotate  15*z translate< -8.15,-5.025,2.8> texture {pigment {radial} finish {ambient 0 diffuse 0.45 specular 1}}}
  scale 0.1
  rotate 90*x
} 
 
 
